# -*- coding: utf-8 -*-


from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [("perimeter", "0002_auto_20150611_1234")]

    operations = [
        migrations.AlterField(
            model_name="accesstokenuse",
            name="user_email",
            field=models.EmailField(
                max_length=254, verbose_name="Token used by (email)"
            ),
        )
    ]



from django.contrib.admin import site, ModelAdmin

from .models import AccessToken, AccessTokenUse


class AccessTokenAdmin(ModelAdmin):

    raw_id_fields = ("created_by",)
    list_display = ("token", "expires_on", "is_active", "created_at", "created_by")
    readonly_fields = ("created_at", "updated_at")

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        """
        Presets the 'created_by' field to the current user.

        Taken from http://stackoverflow.com/a/5633217
        """
        if db_field.name == "created_by":
            kwargs["initial"] = request.user.id
        return super(AccessTokenAdmin, self).formfield_for_foreignkey(
            db_field, request, **kwargs
        )


site.register(AccessToken, AccessTokenAdmin)


class AccessTokenUseAdmin(ModelAdmin):

    list_display = ("token", "expires_on", "timestamp", "client_ip")
    readonly_fields = ("timestamp", "client_user_agent", "client_ip")
    raw_id_fields = ("token",)

    def expires_on(self, obj):
        return obj.token.expires_on

    expires_on.short_description = "Token Expires"


site.register(AccessTokenUse, AccessTokenUseAdmin)



# -*- coding: utf-8 -*-
"""Management command to create a new AccessToken."""
import datetime

from django.core.management.base import BaseCommand
from django.utils.timezone import now
from django.db.utils import IntegrityError

from perimeter.models import AccessToken
from perimeter.settings import PERIMETER_DEFAULT_EXPIRY


class Command(BaseCommand):

    help = "Create a perimeter access token."

    def add_arguments(self, parser):
        parser.add_argument(
            "-t",
            "--token",
            action="store",
            dest="token",
            help="User supplied token (max 10 chars)",
        )
        parser.add_argument(
            "-e",
            "--expires",
            type=int,
            action="store",
            dest="expires",
            help="Expires value (in days)",
        )

    def handle(self, *args, **options):

        has_expires = options.get("expires") is not None
        days = options.get("expires") or PERIMETER_DEFAULT_EXPIRY
        token = options.get("token") or AccessToken.random_token_value()
        expires_on = (now() + datetime.timedelta(days=days)).date()
        try:
            access_token = AccessToken.objects.create_access_token(
                token=token, expires_on=expires_on
            )
            self.stdout.write(
                'Created new access token: "{}" (expires {})'.format(
                    access_token.token, access_token.expires_on
                )
            )
        except IntegrityError:
            access_token = AccessToken.objects.get(token=token)
            if has_expires:
                self.stdout.write("Extending existing token")
                access_token.expires_on = expires_on
                access_token.save()
                self.stdout.write("Token extended: {}".format(access_token))
            else:
                self.stdout.write(
                    "Token exists already - please use --expires option to extend."
                )



"""
Middleware component of Perimeter app - checks all incoming requests for a
valid token. See Perimeter docs for more details.
"""
from urllib.parse import urlencode

from django.core.exceptions import MiddlewareNotUsed, ImproperlyConfigured
from django.http import HttpResponseRedirect
from django.urls import reverse
from django.utils.deprecation import MiddlewareMixin

from .models import AccessToken
from .settings import (
    HTTP_X_PERIMETER_TOKEN,
    PERIMETER_SESSION_KEY,
    PERIMETER_ENABLED,
    PERIMETER_BYPASS_FUNCTION as bypass_perimeter,
)


def check_middleware(request):
    """Check that Session middleware is installed."""
    if not hasattr(request, "session"):
        raise ImproperlyConfigured(
            "Missing session attribute - please check MIDDLEWARE_CLASSES for "
            "'django.contrib.sessions.middleware.SessionMiddleware'."
        )


def get_request_token(request):
    """Extract token string from HTTP header or querystring."""
    check_middleware(request)
    return request.META.get(HTTP_X_PERIMETER_TOKEN, None) or request.session.get(
        PERIMETER_SESSION_KEY, None
    )


def get_access_token(request):
    """Returns AccessToken if found else EmptyToken."""
    token = get_request_token(request)
    # NB this method implements caching, so is more performant
    # than the straight get() alternative
    return AccessToken.objects.get_access_token(token)


class PerimeterAccessMiddleware(MiddlewareMixin):
    """
    Middleware used to detect whether user can access site or not.

    This middleware will be disabled if the PERIMETER_ENABLED setting does not
    exist in django settings, or is False.
    """

    def __init__(self, *args, **kwargs):
        """
        Disable middleware if PERIMETER_ENABLED setting not True.

        Raises MiddlewareNotUsed exception if the PERIMETER_ENABLED setting
        is not True - this is used by Django framework to remove the middleware.
        """
        if PERIMETER_ENABLED is False:
            raise MiddlewareNotUsed()
        super(PerimeterAccessMiddleware, self).__init__(*args, **kwargs)

    def process_request(self, request):
        """Check user session for token."""
        if bypass_perimeter(request):
            return None

        if get_access_token(request).is_valid:
            return None

        # redirect to the gateway for validation,
        qstring = urlencode({"next": request.get_full_path()})
        return HttpResponseRedirect(reverse("perimeter:gateway") + "?" + qstring)



try:
    from django.urls import re_path
except ImportError:
    from django.conf.urls import url as re_path

from . import views

app_name = "perimeter"

urlpatterns = [re_path(r"^gateway/", views.gateway, name="gateway")]



# -*- coding: utf-8 -*-
"""Management command to list all active tokens."""
from django.core.management.base import BaseCommand

from perimeter.models import AccessToken


class Command(BaseCommand):

    help = "List all active tokens."

    def handle(self, *args, **options):

        self.stdout.write("Listing all tokens:")
        for token in AccessToken.objects.all():
            self.stdout.write(str(token))



from urllib.parse import unquote

from django.http import HttpResponseRedirect
from django.shortcuts import render
from django.urls import reverse, resolve, Resolver404

from .forms import UserGatewayForm, TokenGatewayForm
from .settings import PERIMETER_REQUIRE_USER_DETAILS


def resolve_return_url(return_url):
    """Resolve a URL to confirm that it's valid.

    Before redirecting to a return_url, use the resolve function
    to confirm that it matches a valid view function.

    If the resolver can't find the function, return the only url we
    know exists - perimeter:gateway

    """
    path = None
    if return_url:
        return_url = unquote(return_url)
        # Path with a query string will not resolve, so we strip it for the check.
        path = return_url.split("?")[0]

    try:
        resolve(path)
        return return_url
    except Resolver404:
        return reverse("perimeter:gateway")


def gateway(request, template_name="perimeter/gateway.html"):
    """Display gateway form and process access requests.

    When the PerimeterAccessMiddleware catches an unvalidated
    user request they will redirect to this page.

    """
    # the form to use is based on whether we want user details or not.
    klass = UserGatewayForm if PERIMETER_REQUIRE_USER_DETAILS else TokenGatewayForm

    if request.method == "GET":
        form = klass()

    elif request.method == "POST":
        form = klass(request.POST)
        if form.is_valid():
            form.save(request)
            return HttpResponseRedirect(resolve_return_url(request.GET.get("next")))

    return render(request, template_name, {"form": form})



from distutils.version import StrictVersion
from os import path

import django

DJANGO_VERSION = StrictVersion(django.get_version())

# set the django DEBUG option
DEBUG = True

PERIMETER_ENABLED = True

ROOT_URLCONF = "test_app.urls"

# this isn't used, but Django likes having something here for running the tests
DATABASES = {"default": {"ENGINE": "django.db.backends.sqlite3", "NAME": "delme"}}

# NB - this is good for local testing only
DEFAULT_FILE_STORAGE = "django.core.files.storage.FileSystemStorage"
STATICFILES_STORAGE = "django.contrib.staticfiles.storage.StaticFilesStorage"
STATIC_URL = "/static/"
STATIC_ROOT = path.abspath(path.join(path.dirname(__file__), "..", "static"))

INSTALLED_APPS = (
    "django.contrib.contenttypes",
    "django.contrib.auth",
    "django.contrib.admin",
    "django.contrib.messages",
    "django.contrib.sessions",
    "django.contrib.staticfiles",
    "perimeter",
    "test_app",
)

ACTUAL_MIDDLEWARE_CLASSES = [
    "django.middleware.common.CommonMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "perimeter.middleware.PerimeterAccessMiddleware",
]

if DJANGO_VERSION < StrictVersion("1.10.0"):
    MIDDLEWARE_CLASSES = ACTUAL_MIDDLEWARE_CLASSES
else:
    MIDDLEWARE = ACTUAL_MIDDLEWARE_CLASSES

SECRET_KEY = "something really, really hard to guess goes here."

LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "formatters": {"simple": {"format": "%(levelname)s %(message)s"}},
    "handlers": {
        "console": {
            "level": "DEBUG",
            "class": "logging.StreamHandler",
            "formatter": "simple",
        }
    },
    "loggers": {"": {"handlers": ["console"], "propagate": True, "level": "INFO"}},
}

PROJECT_DIR = path.abspath(path.join(path.dirname(__file__)))

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [path.join(PROJECT_DIR, "templates")],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ]
        },
    }
]



import datetime

from django.core.exceptions import ValidationError
from django.test import TestCase, RequestFactory
from django.utils.timezone import now

from ..forms import TokenGatewayForm, UserGatewayForm
from ..models import AccessToken, AccessTokenUse

YESTERDAY = now().date() - datetime.timedelta(days=1)


class BaseGatewayFormTests(TestCase):
    def get_request(self, payload):
        request = self.factory.post(
            "/", data=payload, REMOTE_ADDR="127.0.0.1", HTTP_USER_AGENT="test_agent"
        )
        request.session = {}
        return request

    def get_form(self, klass, payload):
        request = self.get_request(payload)
        form = klass(request.POST)
        return form

    def setUp(self):
        self.factory = RequestFactory()
        self.payload = {"token": "test"}
        self.token = AccessToken(token="test").save()


class TokenGatewayFormTests(BaseGatewayFormTests):
    def test_post_valid_token(self):
        form = self.get_form(TokenGatewayForm, self.payload)
        self.assertTrue(form.is_valid())
        self.assertEqual(form.token, self.token)
        # test with user info missing
        payload = {"token": self.payload["token"]}
        form = self.get_form(TokenGatewayForm, payload)
        self.assertTrue(form.is_valid())

    def test_clean_inactive_token(self):
        form = self.get_form(TokenGatewayForm, self.payload)
        self.token.is_active = False
        self.token.save(update_fields=["is_active"])
        self.assertFalse(form.is_valid())
        self.assertRaises(ValidationError, form.clean_token)

    def test_clean_expired_token(self):
        form = self.get_form(TokenGatewayForm, self.payload)
        self.token.expires_on = YESTERDAY
        self.token.save(update_fields=["expires_on"])
        self.assertFalse(form.is_valid())
        self.assertRaises(ValidationError, form.clean_token)

    def test_no_matching_token(self):
        form = self.get_form(TokenGatewayForm, self.payload)
        AccessToken.objects.all().delete()
        self.assertFalse(form.is_valid())
        self.assertRaises(ValidationError, form.clean_token)

    def test_save(self):
        request = self.get_request(self.payload)
        form = self.get_form(TokenGatewayForm, self.payload)
        self.assertTrue(form.is_valid())
        au = form.save(request)
        self.assertTrue(AccessTokenUse.objects.get(), au)
        self.assertIsNone(au.user_email)
        self.assertIsNone(au.user_name)
        self.assertEqual(au.token, self.token)
        self.assertEqual(au.client_ip, "127.0.0.1")
        self.assertEqual(au.client_user_agent, "test_agent")


class UserGatewayFormTests(BaseGatewayFormTests):
    def setUp(self):
        super(UserGatewayFormTests, self).setUp()

    def test_clean_email(self):
        form = self.get_form(UserGatewayForm, self.payload)
        self.assertFalse(form.is_valid())
        self.payload["email"] = "fred@example.com"
        self.payload["name"] = "Fred"
        form = self.get_form(UserGatewayForm, self.payload)
        self.assertTrue(form.is_valid())

    def test_save(self):
        request = self.get_request(self.payload)
        form = self.get_form(TokenGatewayForm, self.payload)
        self.assertTrue(form.is_valid())
        au = form.save(request)
        self.assertTrue(AccessTokenUse.objects.get(), au)
        self.assertIsNone(au.user_email)
        self.assertIsNone(au.user_name)
        self.assertEqual(au.token, self.token)
        self.assertEqual(au.client_ip, "127.0.0.1")
        self.assertEqual(au.client_user_agent, "test_agent")



from django import forms
from django.core.exceptions import ValidationError

from .models import AccessToken
from .settings import PERIMETER_SESSION_KEY


class TokenGatewayForm(forms.Form):

    """Form used to process a perimeter request."""

    token = forms.CharField(required=True, max_length=100)

    def clean_token(self):
        """Validate the token against existing tokens."""
        try:
            _token = self.cleaned_data.get("token")
            self.token = AccessToken.objects.get(token=_token)
            if self.token.is_valid:
                return _token
            if self.token.has_expired:
                raise ValidationError("Token has expired", code="expired")
            if not self.token.is_active:
                raise ValidationError("Token is inactive", code="invalid")
        except AccessToken.DoesNotExist:
            raise ValidationError("Token not found", code="invalid")

    def save(self, request):
        """Create a new AccessTokenUse object from the form."""
        if getattr(self, "token", None) is None:
            raise ValueError("Form token attr is not set")
        request.session[PERIMETER_SESSION_KEY] = self.token.token
        return self.token.record(
            user_email=self.cleaned_data.get("email"),
            user_name=self.cleaned_data.get("name"),
            client_ip=request.META.get("REMOTE_ADDR", "unknown"),
            client_user_agent=request.META.get("HTTP_USER_AGENT", "unknown"),
        )


class UserGatewayForm(TokenGatewayForm):

    """Form used to process a perimeter request with user info."""

    email = forms.EmailField(required=True)
    name = forms.CharField(required=True, max_length=100)



from django.contrib import admin

try:
    from django.urls import re_path, include
except ImportError:
    from django.conf.urls import url as re_path, include
from django.views.generic import TemplateView

import perimeter.urls

admin.autodiscover()

urlpatterns = [
    re_path(r"^$", TemplateView.as_view(template_name="index.html"), name="homepage"),
    re_path(r"^admin/", admin.site.urls),
    re_path(r"^perimeter/", include(perimeter.urls)),
]



import datetime
import random

from django.conf import settings
from django.core.cache import cache
from django.db import models
from django.db.models.signals import post_save, post_delete
from django.dispatch import receiver
from django.utils import timezone

from .settings import PERIMETER_DEFAULT_EXPIRY


def default_expiry():
    """Return the default expiry date."""
    return (timezone.now() + datetime.timedelta(days=PERIMETER_DEFAULT_EXPIRY)).date()


class EmptyToken(object):
    """Token-like object that will always return is_valid() == False.

    EmptyToken objects are a bit like Django's AnonymousUser model -
    they return an object that can be used like an AccessToken but that
    is always invalid.
    """

    @property
    def is_valid(self):
        return False


class AccessTokenManager(models.Manager):
    """Custom model manager for AccessTokens."""

    def create_access_token(self, **kwargs):
        """Create a new AccessToken with a random token value."""
        # NB there is a theoretical token clash exception here,
        # when the random_token_value function returns an existing
        # token value, but it is considered so unlikely as to be
        # acceptable.
        kwargs["token"] = kwargs.get("token", AccessToken.random_token_value())
        kwargs["expires_on"] = kwargs.get("expires_on", default_expiry())
        return AccessToken(**kwargs).save()

    def get_access_token(self, token_value):
        """Fetch an AccessToken, return EmptyToken if not found.

        This method is cache-aware, and will check the cache first,
        re-filling it if empty.

        Args:
            token - string, the token value to look up.

        """
        cache_key = AccessToken.get_cache_key(token_value)
        token = cache.get(cache_key)
        if token is None:
            try:
                token = self.get(token=token_value)
                cache.set(token.cache_key, token, token.seconds_to_expiry)
                return token
            except AccessToken.DoesNotExist:
                return EmptyToken()
        else:
            return token


class AccessToken(models.Model):
    """
    A token that allows a user entry to the site via Perimeter.
    """

    token = models.CharField(max_length=50, unique=True)
    is_active = models.BooleanField(default=True)
    # NB pass in a callable, not the result of the callable, see:
    # http://stackoverflow.com/a/29549675/45698
    expires_on = models.DateField(default=default_expiry)
    created_by = models.ForeignKey(
        settings.AUTH_USER_MODEL, on_delete=models.CASCADE, null=True
    )
    created_at = models.DateTimeField()
    updated_at = models.DateTimeField()

    objects = AccessTokenManager()

    def __str__(self):
        return self.token

    @classmethod
    def random_token_value(cls):
        """Generate a random token value."""
        return "".join(
            random.sample(
                population=list(
                    "abcdefghijklmnopqrstuvwxyz"
                    "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
                    "1234567890"
                ),
                k=cls._meta.get_field("token").max_length,
            )
        )

    @classmethod
    def get_cache_key(cls, token_value):
        return "%s.%s-%s" % (cls.__module__, cls.__name__, token_value)

    def save(self, *args, **kwargs):
        "Sets the created_at timestamp."
        self.updated_at = timezone.now()
        self.created_at = self.created_at or self.updated_at
        super(AccessToken, self).save(*args, **kwargs)
        return self

    @property
    def cache_key(self):
        """Return object cache key (from get `get_cache_key`)."""
        return AccessToken.get_cache_key(self.token)

    @property
    def seconds_to_expiry(self):
        """Return the number of seconds till expiry (used for caching)."""
        expires_at = datetime.datetime.combine(self.expires_on, datetime.time.min)
        if settings.USE_TZ and timezone.is_naive(expires_at):
            expires_at = timezone.make_aware(
                expires_at, timezone.get_current_timezone()
            )
        elif not settings.USE_TZ and timezone.is_aware(expires_at):
            expires_at = timezone.make_naive(expires_at)
        return int((expires_at - timezone.now()).total_seconds())

    @property
    def has_expired(self):
        """Return True if the token has passed expiry date."""
        return self.expires_on < datetime.date.today()

    @property
    def is_valid(self):
        """Return True if the token is active and has not expired."""
        return self.is_active and not self.has_expired

    def record(
        self, user_email, user_name, client_ip="unknown", client_user_agent="unknown"
    ):
        """Record the fact that someone has used the token."""
        atu = AccessTokenUse(
            token=self,
            user_email=user_email,
            user_name=user_name,
            client_ip=client_ip,
            client_user_agent=client_user_agent,
        )
        atu.save()
        return atu


@receiver(post_save, sender=AccessToken)
def on_save_access_token(sender, instance, **kwargs):
    """Update saved object in cache if is_valid, else delete."""
    cache.set(instance.cache_key, instance, instance.seconds_to_expiry)


@receiver(post_delete, sender=AccessToken)
def on_delete_access_token(sender, instance, **kwargs):
    """Remove deleted object from cache."""
    cache.delete(instance.cache_key)


class AccessTokenUse(models.Model):
    """Audit record used to log whenever an access token is used."""

    token = models.ForeignKey(AccessToken, on_delete=models.CASCADE)
    user_email = models.EmailField(
        verbose_name="Token used by (email)", blank=True, null=True
    )
    user_name = models.CharField(
        max_length=100, verbose_name="Token used by (name)", blank=True, null=True
    )
    client_ip = models.CharField(max_length=15, verbose_name="IP address", blank=True)
    client_user_agent = models.TextField(verbose_name="Client User Agent", blank=True)
    timestamp = models.DateTimeField()

    def __str__(self):
        return "'%s' used %s" % (self.token.token, self.timestamp)

    def save(self, *args, **kwargs):
        "Set the timestamp and save the object."
        if "update_fields" not in kwargs:
            self.timestamp = self.timestamp or timezone.now()
        super(AccessTokenUse, self).save(*args, **kwargs)
        return self



from os import path, pardir, chdir
from setuptools import setup, find_packages

README = open(path.join(path.dirname(__file__), "README.rst")).read()

# allow setup.py to be run from any path
chdir(path.normpath(path.join(path.abspath(__file__), pardir)))

setup(
    name="django-perimeter",
    version="0.12",
    packages=find_packages(),
    include_package_data=True,
    license="MIT",
    description="Site-wide perimeter access control for Django projects.",
    long_description=README,
    url="https://github.com/yunojuno/django-perimeter",
    author="YunoJuno",
    author_email="code@yunojuno.com",
    maintainer="YunoJuno",
    maintainer_email="code@yunojuno.com",
    install_requires=["Django>=1.11"],
    classifiers=[
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Django :: 1.11",
        "Framework :: Django :: 2.0",
        "Framework :: Django :: 2.1",
        "Framework :: Django :: 2.2",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    ],
)



from urllib.parse import urlparse

from django.contrib.auth.models import User, AnonymousUser
from django.core.exceptions import ImproperlyConfigured
from django.test import TestCase, RequestFactory, override_settings
from django.urls import reverse, resolve

from ..middleware import (
    PerimeterAccessMiddleware,
    bypass_perimeter,
    get_access_token,
    get_request_token,
    HTTP_X_PERIMETER_TOKEN,
    PERIMETER_SESSION_KEY,
)
from ..models import AccessToken, EmptyToken


@override_settings(PERIMETER_ENABLED=True)
class PerimeterMiddlewareTests(TestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.request = self.factory.get("/")
        # spoof the Auth and Session middleware
        self.request.user = User()
        self.request.session = {}
        self.middleware = PerimeterAccessMiddleware()

    def _assertRedirectsToGateway(self, request, query=None):
        query = query or "next=%2F"
        # NB assertRedirects doesn't work!
        resp = self.middleware.process_request(request)
        self.assertEqual(resp.status_code, 302)
        # use a resolver to strip off any quesrystring params
        resolver = resolve(resp.url)
        self.assertEqual(resolver.url_name, "gateway")
        self.assertEqual(resolver.namespace, "perimeter")
        self.assertEqual(urlparse(resp.url).query, query)

    def test_bypass_perimeter_default(self):
        """Perimeter login urls excluded."""
        request = self.factory.get("/")
        self.assertFalse(bypass_perimeter(request))
        request = self.factory.get(reverse("perimeter:gateway"))
        self.assertTrue(bypass_perimeter(request))

    def test_get_request_token_session(self):
        at = AccessToken.objects.create_access_token()
        self.request.session[PERIMETER_SESSION_KEY] = at.token
        self.assertEqual(get_request_token(self.request), at.token)

    def test_get_request_token_http_header(self):
        at = AccessToken.objects.create_access_token()
        request = self.factory.get("/", HTTP_X_PERIMETER_TOKEN=at.token)
        request.session = {}
        self.assertEqual(get_request_token(request), at.token)

    def test_get_access_token(self):
        at = AccessToken.objects.create_access_token()
        self.request.session[PERIMETER_SESSION_KEY] = at.token
        self.assertEqual(get_access_token(self.request), at)

    def test_get_request_token_empty(self):
        token = get_request_token(self.request)
        self.assertIsNone(token)

    def test_get_access_token_empty(self):
        token = get_access_token(self.request)
        self.assertIsInstance(token, EmptyToken)

    def test_missing_session(self):
        """Missing request.session should raise AssertionError."""
        del self.request.session
        self.assertRaises(ImproperlyConfigured, get_request_token, self.request)

    def test_missing_token(self):
        """AnonymousUser without a token should be denied."""
        self.request.user = AnonymousUser()
        self._assertRedirectsToGateway(self.request)

    def test_invalid_token(self):
        self.request.user = AnonymousUser()
        self.request.session["token"] = "foobar"
        self._assertRedirectsToGateway(self.request)

    def test_valid_token(self):
        """AnonymousUser with a valid session token should pass through."""
        AccessToken(token="foobar").save()
        self.request.user = AnonymousUser()
        self.request.session["token"] = "foobar"
        self._assertRedirectsToGateway(self.request)

    def test_perimeter_token_header(self):
        """Test that the X-Perimeter-Token header works."""
        AccessToken(token="foobar").save()
        self.request.user = AnonymousUser()
        self._assertRedirectsToGateway(self.request)
        self.request.META["HTTP_X_PERIMETER_TOKEN"] = "foobar"
        self.middleware.process_request(self.request)

    def test_next_query_string_set(self):
        """Check `next` query string param is properly encoded"""

        # Simple path
        request = self.factory.get("/somepath/")
        request.user = AnonymousUser()
        request.session = {}
        self._assertRedirectsToGateway(request, query="next=%2Fsomepath%2F")

        # Path with query string
        request = self.factory.get("/somepath/?important=param")
        request.user = AnonymousUser()
        request.session = {}
        self._assertRedirectsToGateway(
            request, query="next=%2Fsomepath%2F%3Fimportant%3Dparam"
        )
